print("import: 'tokenizers'")
import tokenizers

print("import: 'tokenizers.models'")
import tokenizers.models

print("import: 'tokenizers.decoders'")
import tokenizers.decoders

print("import: 'tokenizers.normalizers'")
import tokenizers.normalizers

print("import: 'tokenizers.pre_tokenizers'")
import tokenizers.pre_tokenizers

print("import: 'tokenizers.processors'")
import tokenizers.processors

print("import: 'tokenizers.trainers'")
import tokenizers.trainers

print("import: 'tokenizers.implementations'")
import tokenizers.implementations

print("import: 'tokenizers.tools'")
import tokenizers.tools

print("import: 'huggingface_hub'")
import huggingface_hub

